const jsonLogic = require('json-logic-js')

jsonLogic.add_operation('base64_encode', function (data) {
  return Buffer.from(data).toString('base64')
})

jsonLogic.add_operation('base64_decode', function (data) {
  return Buffer.from(data, 'base64').toString()
})

const { LogicEngine } = require('../')

const x = new LogicEngine()

x.addMethod('base64_encode', i => {
  return Buffer.from(i).toString('base64')
})

x.addMethod('base64_decode', i => {
  return Buffer.from(i, 'base64').toString()
})

const f = x.build({
  base64_decode: { base64_encode: { cat: ['Hello,', { var: '' }, '!'] } }
}) // ?

function repeat (name, times, func) {
  console.time(name)
  for (let i = 0; i < times; i++) { func(i) }
  console.timeEnd(name)
}

repeat('Base64 Encode', 1e6, f)

repeat('Base64 Manual', 1e6, (i) => {
  return Buffer.from(Buffer.from(`Hello, ${i}!`).toString('base64'), 'base64').toString()
})

repeat('Base64 json-logic-js', 1e6, (i) => {
  return jsonLogic.apply({
    base64_decode: { base64_encode: { cat: ['Hello,', { var: '' }, '!'] } }
  }, i)
})
